#include <glib-object.h>
#include <string.h>
#include <gmodule.h>
#include <gtk/gtk.h>
#include <pango.h>

#include "prop-util.h"
#include "xml.h"
#include "widget.h"

GType wps_get_type_from_name(char *name)
{
	static GModule *allsymbols = NULL;
	GType (*get_type)();
	GType type = 0;
	GString *get_type_string;
	gint i;
	gboolean begin = TRUE;
	
	get_type_string = g_string_new(name);
	
	for (i = 0; i < get_type_string->len; i++) {
		if (g_ascii_isupper(get_type_string->str[i])) {
			if (!begin) {
				get_type_string = g_string_insert_c(get_type_string, i, '_');
				i++;
			}
			begin = FALSE;
			get_type_string->str[i] = g_ascii_tolower(get_type_string->str[i]);						
		}		
	}
	
	get_type_string = g_string_append(get_type_string, "_get_type");
	
	type = g_type_from_name(get_type_string->str);
	
	if (!type) {
		if (!allsymbols)
			allsymbols = g_module_open(NULL, 0);

		if (g_module_symbol(allsymbols, get_type_string->str, (gpointer)&get_type))
			type = get_type();
		else
			g_warning("Can't find symbol\n");
	}
	
	if (type == 0)
		g_warning("Could not get the type from %s", name);
		
	return type;
}

void wps_set_value(GParamSpec *param_spec, PropInfo *prop_info, GValue *value)
{
	gint ret_int;
	guint ret_uint;
	gdouble ret_double;
	
	if (G_IS_PARAM_SPEC_STRING(param_spec)) {
		g_value_init(value, G_TYPE_STRING);
		g_value_set_string(value, prop_info->value->str);
	}
	else if (G_IS_PARAM_SPEC_UINT(param_spec)){
		g_value_init(value, G_TYPE_UINT);
				
		if (!strncmp(prop_info->value->str, "0x", 2))
			ret_uint = g_ascii_strtoll(prop_info->value->str, NULL, 16);
		else
			ret_uint = g_ascii_strtoll(prop_info->value->str, NULL, 10);
			
		g_value_set_uint(value, ret_uint);
	}
	else if (G_IS_PARAM_SPEC_INT(param_spec)) {
		g_value_init(value, G_TYPE_INT);
				
		if (!strncmp(prop_info->value->str, "0x", 2))
			ret_int = g_ascii_strtoll(prop_info->value->str, NULL, 16);
		else
			ret_int = g_ascii_strtoll(prop_info->value->str, NULL, 10);
			
		g_value_set_int(value, ret_int);
	}
	else if (G_IS_PARAM_SPEC_DOUBLE(param_spec)) {
		g_value_init(value, G_TYPE_DOUBLE);
		ret_double = g_ascii_strtod(prop_info->value->str, NULL);
		g_value_set_double(value, ret_double);
	}
	else if (G_IS_PARAM_SPEC_BOOLEAN(param_spec)){
		g_value_init(value, G_TYPE_BOOLEAN);
				
		if (!g_strcmp0(prop_info->value->str, "TRUE"))
			g_value_set_boolean(value, TRUE);
		else if (!g_strcmp0(prop_info->value->str, "FALSE"))
			g_value_set_boolean(value, FALSE);
	}
	else if (G_IS_PARAM_SPEC_OBJECT(param_spec)) {
		if (!g_strcmp0(param_spec->name, "adjustment")) {
			GtkAdjustment *gtk_adjustment;
			gdouble min = 0.0;
			gdouble max = 100.0;
			gdouble step = 1;
			
			gtk_adjustment = (GtkObject *)gtk_adjustment_new(0, min, max, step, 0, 0);
			g_value_init(value, G_TYPE_OBJECT);
			g_value_set_object(value, gtk_adjustment);
		}
		else if (!g_strcmp0(param_spec->name, "pixbuf")) {
			GdkPixbuf *pixbuf;
			
			pixbuf = gdk_pixbuf_new_from_file(prop_info->value->str, (GError **)NULL);
			g_value_init(value, G_TYPE_OBJECT);
			g_value_set_object(value, pixbuf);
			
			g_warning("PIXBUF\n");
		}
	}
	else if (G_IS_PARAM_SPEC_BOXED(param_spec)) {
		if (!g_strcmp0(param_spec->name, "points")) {
			GooCanvasPoints *points;
			
			points = goo_canvas_points_new (5);
			points->coords[0] = 0.0;
			points->coords[1] = 0.0;
			points->coords[2] = 10.0;
			points->coords[3] = 30.0;
			points->coords[4] = 30.0;
			points->coords[5] = 30.0;
			points->coords[6] = 30.0;
			points->coords[7] = 50.0;
			points->coords[8] = 100.0;
			points->coords[9] = 100.0;
			
			g_value_init(value, param_spec->value_type);
			g_value_set_boxed(value, points);
		}
	}
	else
		g_warning("Unknown property type\n");
	
}

void wps_position_prop_correct(GList *template_widget_list, TemplateInUse *templ_use)
{
	GList *last_widget = NULL;
	WPSWidget *widget;
	
	
	for (last_widget = g_list_last(template_widget_list); last_widget != NULL; last_widget = g_list_previous(last_widget)) {
		widget = last_widget->data;
		goo_canvas_item_translate(widget->goo_item, templ_use->xalign, templ_use->yalign);
	} 
}

void widget_pango_font_set(GtkWidget *widget, StyleInfo *style)
{
	PangoFontDescription *font_desc;
	
	font_desc = pango_font_description_copy(widget->style->font_desc);
	pango_font_description_set_size(font_desc, style->font_size * PANGO_SCALE);
	gtk_widget_modify_font(widget, font_desc);
	pango_font_description_free(font_desc);
}

GParameter *set_gparam(GObjectClass *klass, GArray *prop_info_array, gint len)
{
	GParamSpec **param_spec;
	GParameter *gparam;
	guint nprop;
	gint i, j;
	
	param_spec = g_object_class_list_properties(klass, &nprop);
	gparam = g_new0(GParameter, len);
	
	for (i = 0; i < nprop; i++) {
		for (j = 0; j < len; j++) {
			PropInfo *prop;
			
			prop = g_array_index(prop_info_array, PropInfo*, j);
			
			if (g_strcmp0(param_spec[i]->name, prop->name->str) == 0) {
				GValue *value;
				gparam[j].name = g_strdup(prop->name->str);
				
				value = g_new0(GValue, 1);
				wps_set_value(param_spec[i], prop, value);
				gparam[j].value = *value;
				g_free(value);				
			}	
		}			
	}
	
	return gparam;
}
